#ifndef MAINDLG_H
#define MAINDLG_H

#include <QMainWindow>

#include "includes.h"

#include "uart.h"
#include "ps.h"
#include "oambaseparam.h"
#include "oamconfigparam.h"
#include "oaminstallparam.h"
#include "oamnetworkparam.h"
#include "oamsyncparam.h"
#include "oamsynctimer.h"
#include "oamupgrade.h"
#include "oambtsparam.h"
#include "sysconfig.h"
#include "httpreq.h"

#define MAINDLG_DEFAULT_PACKET_NO -100


QT_BEGIN_NAMESPACE
namespace Ui {
class MainDlg;
}
QT_END_NAMESPACE

class MainDlg : public QMainWindow
{
    Q_OBJECT

public:
    MainDlg(QWidget *parent = nullptr);
    ~MainDlg();

    HttpReq *m_pHttpReq;
    int m_dwDeviceType;
    int m_dwTabIndex;

    void setProjectId(quint32 dwProjectId);
    quint32 getPorjectId();
    void mainDlgSetBtsId(quint32 dwBtsId);
    int getDeviceType();//查询设备类型
    void stopUpgradeTimer();
    void configHttpReq();

private:
    Ui::MainDlg *ui;
    QTimer *m_pTimer;
    QTimer *m_pUpgradeTimer;
    qint8 m_byPacketNo;
    quint8 m_abyTmpData[PS_FRAME_MAX_LEN];

    Uart *m_pUart;//串口对象
    Ps *m_objPs;//协议栈对象

    sysconfig *m_pSysConfig;//系统参数类对象
    quint32 m_dwProjectId;//项目编号
    quint32 m_dwBtsId;//基站ID

    quint8 m_byHttpReqConfigOk;//网络模块配置OK

    OamBaseParam *m_pOamBaseParam;
    OamConfigParam *m_pOamConfigParam;
    OamInstallParam *m_pOamInstallParam;
    OamNetworkParam *m_pOamNetworkParam;
    OamSyncParam *m_pOamSyncParam;
    oamsynctimer *m_pOamSyncTimer; //同步定时
    OamUpgrade *m_pOamUpgrade;//升级对话框
    OamBtsParam *m_pOamBtsParam;//基站标识



    //私有成员函数
    void closeEvent(QCloseEvent *event);
    void Init();
    void initUi();
    void configSignalAndSlot();

    void getUartNameList();
    void openUart();
    void closeUart();
    qint64 sendDataToUart(QByteArray dataList);

signals:
    void signalNoticeUpgradeOk(quint16 wTag);//升级OK信号
    void signalNoticeUpgradeFail();//升级失败信号
    void signalNoticeUpgradeTimeout();//升级超时

public slots:
    void slotDeviceTypeChanged(int index);
    void slotTabSwitch(int index);
    void slotReceiveDataFromUart(QByteArray dataList, quint16 wLen);
    void slotOamSendQueryMsg(quint8 *pbyData, quint16 wLen);//发送查询报文
    void slotOamSendConfigMsg(quint8 *pbyData, quint16 wLen, quint16 wCmd);//发送配置报文
    void slotOamSendUpgradeMsg(quint8 *pbyData, quint8 byCmdType, quint16 dataLen);//发送升级报文

    void slotUploadBtsBaseParamAdd(QString strApi, QJsonObject jsonObject);//上传基站基本参数-增加
    void slotUploadBtsBaseParamUpdate(QString strApi, QJsonObject jsonObject);//发送上传基站参数

    void PsUlRespSignaQueryOkSlot(quint8 *pbyData, quint16 wLen);//协议栈上行查询响应成功信号槽函数
    void PsUlRespSignaQueryFailSlot();//协议栈上行查询响应失败信号槽函数
    void PsUlRespSignalConfigOkSlot(quint8 *pbyData, quint16 wLen);
    void PsUlRespSignalConfigFailSlot();
    void Timeout2000Slot();
    void Timeout5000Slot();

    void PsUlRespSignalUpgradeOkSlot(quint8 *pbyData);
    void PsUlRespSignalUpgradeFailSlot();
    void slotUpgradeRestart();

};
#endif // MAINDLG_H
